/* ---------------------------------------------

MiMardec - added pin numbers to the output

Mega: 12 servo no pwm on 44,45,46
      24 servo no pwm on 11,12
      36 servo no pwm on 2,3,5
      48 servo no pwm at all
	  0 servo pwm on 2,3,4,5,6,7,8,9,10,11,12,13,44,45,46
 * ------------------------------------------------	  
 * Multifunctional DCC decoder for controlling servo's, accessory's and PWM output
 * Author=Nico Teering
 * Version 6.0 November 2019
 * DCC library and sample coding provided by Mynabay.
 * 
 * This source code or any part of it may NOT be used in any commercial product.
 * Nor it may be modified in any private project without informing the author.
 * 
 * Please mail info@arcomora.com if you have any comment, questions or  
 * suggestions for improvement on this program.
 * -------------------------------------------------------
 * Release Notes 4.0 august 2018
 *
 * Bugs fixed=
 * The restore of servo's at startup to its last position did not work properly (Thanks Ruud Boer)
 * The servo position was not saved in configure mode when a + or - command was the last command before saving.
 * 
 * Changes:
 * Putty speed set to 57600 baud
 * When displaying an adress/pin it is always showed as A<address>/P<pin>
 *   E.g 'Servo on A15/P15 set to 108 degrees' or 'Specify action for servo on A2/P3'
 * Added a generic option to detach a servo or not.
 * Added an option to set startup mode (Last, Config or Normal mode)
 * Added the generic I-command. This command let you change:
 *   the Mardec number, Roco offset, Startup mode, Default servo speed, detach servo option
 * Added the I-command for Accessories. This inverts the output
 * Added better support for MEGA2560. 
 *	 On a MEGA the (logical) pins 14 to 19 controls the (physical) pins 54 to 59
 * The default rotation speed is now part of the generic I-command.
 * Some minor display enhancements.
 *--------------------------------------------
 * Release 4.0a  November 2018
 * bug fix=detach/attach setting not correct=always set to detached
 * -------------------------------------------
 * Release 6.0b December 2019
 * 
 * Added bouncing for servo's
 * Modified Test option for accessories
 * Simplified menu structure for accessories
 * Single One shot also on Up-pulse
 * Improved startup procedure
 * Fix for servo's on port 12-16
 * =================================================================================
*/
#include <EEPROM.h>
#include <Servo.h> 
//#include <TTP229.h>
#include <DCC_Decoder.h>
#define kDCC_INTERRUPT 0
#define kONE_Max 80         // for compliance with Digikeijs DR5000;
//------------------------

const byte MAXTERM=17;
const int MAXDCC=2000;
const int C_INPWAIT=1000;
const int MAXRAND=30000;
//const int MAXRANDS=5000;
const int MINRAND=1;
//const int MINRANDS=5;
//const String wit=  "\e[37m\e[1m";
//const String blauw="\e[36m\e[1m";
//const String geel= "\e[33m\e[1m";
const byte resetval=222;
const byte Val255=255;
const byte MAXSPEED=100;

const byte A_DEFSPEED=1;
const byte A_RMODE=2;
const byte A_RESET=3;
const byte A_OFFSET=4;
const byte A_DECNUM=5;
const byte A_KEEPMODE=6;
const byte A_DETACH=7;
const byte A_SMODE=8;
const byte A_OUTSET=20;
const byte A_PORT=100;
const byte A_GETPUT=30; //ruimte voor één poortitem

const byte T_Access=1;
const byte T_Servo=2;
const byte T_Buddy=3;
const byte T_Frog1=4;
const byte T_Frog2=5;
const byte T_Input=6;

const byte M_SiSteady=1;
const byte M_DuSteady=2;
const byte M_SiFlash=3;
const byte M_DuFlash=4;
const byte M_SiOneShot=5;
const byte M_DuOneShot=6;
const byte M_AnalPWM=7;
const byte M_FirePWM=8;
const byte M_Random=9;
//const byte M_RandShort=10;

struct PortItem
{                           
//                          Servo			Accessory	          Input	  Frog 1	 Frog 2	  Buddy
	byte	type;		  //  2	               1	                6	         4 	     5	       3
	byte	Langle;	      //Lage hoek	    laagste PWM waarde	  delay2		
	byte	Hangle;	      //Hoge hoek	    hoogste PWM waard	  delay3		
	unsigned int address; //DCC adres -->
	boolean	Invert;		  //Inv. servo	    Inv. output		                Inv.	   Inv.	
	char	admin[5];	  //Adm Note -->		
	byte	frogbud;	  //Frog 1	        Buddy pin		    Type(UDB) ServoPort  ServoPort   acc. Pin
	byte	SpeedMode;    //snelheid	    1 t/m 10				
	unsigned int BounceLOW; //BounceLOW
	byte	frog2;		  //Frog 2           
//	byte    Ardpin		  //Arduino pin
	unsigned int RiseOn;  //servonr.   	    Stijg/Aan tijd 			Dcc2
	unsigned int FallOffBH;	//BounceHIGH	    Daal/Uit tijd 			Dcc3
//      									(>0ms;<0sec)	                                  
};
PortItem AllPorts[MAXTERM];

struct PinVar
{
  byte output;              //DCC output              DCC output  
  byte savout;  
  byte swrelay;             //switchangle             on/off state           
  byte angle;               //Current angle                                 
  byte setpoint;            //Destination angle       last output           
  boolean free;              	//Busy or not         flash on/off 
  byte doel;                //for bouncing
  unsigned long lastmillis; //Time at last change
  byte whoek;
  boolean PortWait;
  int DelWait;
  unsigned long InpWait;
//  byte Bounce[4];	  //bouncehoeken
};
PinVar AllVar[MAXTERM];
Servo Servos[12];  //0 t/m 11
byte SerUsed[] = {0,0,0,0,0,0,0,0,0,0,0,0};

byte LastInput[MAXTERM];
byte BounceHL[12][4][2];

#if defined(ARDUINO_AVR_MEGA2560)
const byte p[MAXTERM]={0,3,4,5,6,7,8,9,10,11,12,54,55,56,57,58,59};
//       port=           1 2 3 4 5 6 7 8  9  10 11 12 13 14 15 16 
boolean MEGA=true;
int MAXMEM = 4096;
byte MAXSERVO=12;
//       port=             1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 
const byte pwmports[] = {0,1,1,1,1,1,1,1,1,1,1, 0, 0, 0, 0, 0, 0}; //valid pwm  pin 3,4,5,6,7,8,9,10=port 1,2,3,4,5,6,7,8
unsigned long steptime[8];
byte updown[8];
byte curpwm[8];  //    1 2 3 4 5 6 7 8
const  byte pwm[]={255,0,1,2,3,4,5,6,7};
// UNO
#else 
//	if (cpin>12) cpin=cpin-3 else cpin=cpin-2;
const byte p[MAXTERM] = {0,3,4,5,6,7,8,9,10,11,12,14,15,16,17,18,19};
//       port=             1 2 3 4 5 6 7 8  9  10 11 12 13 14 15 16 
boolean MEGA=false;
int MAXMEM=1024;
byte MAXSERVO=12;  
const  byte pwmports[] = {0,1,0,1,1,0,0,0,0,1,0, 0, 0, 0, 0, 0, 0}; //valid pwm poort 1,3,4,9=pin 3,5,6,11
//       port=              1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 
unsigned long steptime[4];
byte updown[4];
byte curpwm[4];  //    1     3 4                 9
const  byte pwm[]={255,0,255,1,2,255,255,255,255,3};
#endif
//byte t[20]={0,0,0,1,2,3,4,5,6,7, 8, 9,10,0, 11,12,13,14,15,16};
//     pin=0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19

unsigned long waitmil[MAXTERM];
//unsigned long InpWait[MAXTERM];
unsigned long l13mil;
char cmdLH[MAXTERM];    //setting Low or H angle
//bool PortWait[MAXTERM];
boolean Rmode;          //config=false/normal=true
boolean changes=false;  //Wijzigingen?
byte curport;        //Current pin being set
boolean valid;          //Entry valid?
//bool AccInv[MAXTERM];
//String cmdopt;
//boolean notice[MAXTERM];
boolean deta;
byte Smode;
byte defspeed;          //default servo speed
byte offset;            //=1 and =5 for Roco
char gencmd;            // command entered

unsigned long previousMillis;
boolean LED13state = true;
boolean InpSingle;
byte decnum;
boolean nextloop=false;
//unsigned long steptime[MAXTERM];
//byte updown[MAXTERM];
//byte curpwm[MAXTERM];
byte numservo=0;
boolean pinsel = false;  //true if pin selected, false for generic mode 

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void setup()
{
//  for (byte pin=1;pin<MAXTERM;pin++)
//  { 
//	pinMode(p[pin],INPUT_PULLUP);
//	pinMode(p[pin],OUTPUT);
//  }
  pinMode(LED_BUILTIN,OUTPUT);
  StartScreen();
  Readmem();
  if (decnum>100) 	//nieuwe mardec
  {
	  for (int n=0;n<MAXMEM;n++) EEPROM.update(n,Val255);
	  AskNum21();
	  Rmode=false;
  }
//----------------------  setup
  
  if (Rmode) SetupNormal(); else SetupCon();

//  Serial.println("\nResetting Accessories");
  for (byte n=1; n<MAXTERM; n++)    //restore accessories and inputs
  {
    if (AllPorts[n].type==T_Access) //accessory
    {
	  digitalWrite (p[n],AllPorts[n].Invert^LOW);  //uitzetten
	  if (AllPorts[n].frogbud<20)  digitalWrite (p[AllPorts[n].frogbud],AllPorts[n].Invert^LOW);
	  
//      if (AllPorts[n].SpeedMode==M_SiOneShot || AllPorts[n].SpeedMode==6) //one shot always off
	  if (AllPorts[n].SpeedMode==M_SiOneShot) //one shot always off
      {
        AllVar[n].output=0;
        AllVar[n].swrelay=0;
      }
      else
      {
        AllVar[n].output=EEPROM.read(A_OUTSET+n);
        AllVar[n].swrelay=1-AllVar[n].output;	
      }
    }
    if (AllPorts[n].frogbud=='L' && AllPorts[n].type==T_Input) AllPorts[n].frogbud='B';
  }  
  NormalMode();  //eenmalig om alles goed te zetten
  
  for (byte i=0;i<3;i++)
  {
      digitalWrite(LED_BUILTIN,HIGH);
      delay(250);
      digitalWrite(LED_BUILTIN,LOW);
      delay(250);    
  }
//  Serial.println("\nResetting Servo's");
  for (byte n=1; n<MAXTERM; n++)    //restore servo
  {
    if (AllPorts[n].type==T_Servo)   
    {
      numservo++;
      AllVar[n].setpoint = EEPROM.read(A_OUTSET+n);  // lees laatste waarde
      if (AllVar[n].setpoint < AllPorts[n].Langle) AllVar[n].setpoint=AllPorts[n].Langle;
	  else if (AllVar[n].setpoint > AllPorts[n].Hangle) AllVar[n].setpoint=AllPorts[n].Hangle;
	  if (AllPorts[n].RiseOn<0 || AllPorts[n].RiseOn>12) //update van 5.1
	  {
		AllPorts[n].RiseOn=FindFreeServo();
		AllPorts[n].BounceLOW=0; AllPorts[n].FallOffBH=0;
		curport=n; SavePort();
	  }
	  SerUsed[AllPorts[n].RiseOn]=1;
	  if (Rmode)
	  {
		Servos[AllPorts[n].RiseOn].attach(p[n]);
		Servos[AllPorts[n].RiseOn].write(AllVar[n].setpoint);  //hard restore for servo
		;
		Servos[AllPorts[n].RiseOn].detach();
		if (AllPorts[n].frog2<MAXTERM) digitalWrite(p[AllPorts[n].frog2],!AllPorts[AllPorts[n].frog2].Invert); // koppel puntstuk
		ShowHoek(n);
		if (AllVar[n].setpoint==AllPorts[n].Langle)  AllVar[n].output=1;
			else if (AllVar[n].setpoint==AllPorts[n].Hangle) AllVar[n].output=0;
		if (AllPorts[n].Invert) AllVar[n].output=1-AllVar[n].output;
		AllVar[n].savout = AllVar[n].output;
		AllVar[n].angle= AllVar[n].setpoint;
		SwitchRelay(n);
	  }
	  SetBounce(n);
    }
  }

  nextloop=true;
  Serial.println("\nMardec started\n");
  while(Serial.available()) Serial.read();   //Empty inputbuffer1
    
  if (!Rmode) digitalWrite (LED_BUILTIN,HIGH);
} 
// end setup
//--------------------------------------------
void CvtBounce(int bounce, byte HL, byte n)
{
int nr, nrd;
int ten[]={1000,100,10,1};

		nr=bounce;
		for (byte ba=0 ; ba<4 ; ba++)
		{			 
		  BounceHL[AllPorts[n].RiseOn][ba][HL]=0;
		  if (nr>=ten[ba])
		  {
			nrd=nr/ten[ba];
			BounceHL[AllPorts[n].RiseOn][ba][HL]=nrd;
			nr=nr-(nrd*ten[ba]);
//	Serial.println (BounceHL[AllPorts[n].RiseOn][ba][HL]);			
		  }
		} 
}
//----------------------------
void AskNum21()
{
	Serial.println(F("\nEnter number for this MARDEC"));
	if (decnum>100) decnum=AskInteger(-1,1,100); else decnum=AskInteger(decnum,1,100);
	EEPROM.update(A_DECNUM,decnum);

	dwit();
	Serial.print(F("\nThis is MARDEC #"));
	Serial.println(decnum);
	dgeel();
// 	Serial.println(EEPROM.read(A_DECNUM));
	
	Serial.print(F("\nDo you use a ROCO MultiMaus or z/Z21 Y/N [N] ? "));
	if (AskYN()=='Y') offset=5; else offset=1;
	EEPROM.update(A_OFFSET,offset);
	dwit();
	Serial.print(F("Address offset: "));
	if (offset==1) Serial.println(F("No")); else Serial.println(F("Yes"));
	dgeel();
	
	DefaultSpeed();
	
	Serial.print(F("\nDetach servo at end of rotation Y/N [Y] ? "));
	if (AskYN()=='N') deta=false; else deta=true;
	EEPROM.update(A_DETACH,deta);
	dwit();
	Showdeta();
	dgeel();

    Serial.println(F("\nStartup mode: 1=Last (def.),2=Configuration,3=Normal"));
	if (Smode>3 || Smode<1) Smode=1; //nog niet gezet
    Smode=AskInteger(Smode,1,3);
    EEPROM.update(A_SMODE,Smode);	
	EEPROM.update(A_RMODE,false);	
	EEPROM.update(A_KEEPMODE,false);	
	dwit();
	Serial.println("");
	ShowSM();
	dgeel();
}
void ShowSM()
{
	Serial.print ("Startup mode: ");
	if (Smode==1) Serial.println(F("Last\n"));
	else if (Smode==2) Serial.println(F("Configuration\n"));
	else if (Smode==3) Serial.println(F("Normal\n"));
}

void Showdeta()
{
	if (deta) Serial.print(F("Servo's are detached")); else Serial.print(F("Servo's remain attached"));
	Serial.println(F(" at end of rotation"));
}
//--------------------------- Readmem
void Readmem()
{
bool Keepmode;
  
  offset=EEPROM.read(A_OFFSET);
  if (offset==Val255) 
  {
	  offset=1;
	  EEPROM.update(A_OFFSET,offset);
  }
  
  defspeed=EEPROM.read(A_DEFSPEED);
  if (defspeed==Val255) 
  {
	  defspeed=25;
	  EEPROM.update(A_DEFSPEED,defspeed);
  }
  
  decnum=EEPROM.read(A_DECNUM);
  deta=EEPROM.read(A_DETACH);
  Rmode=EEPROM.read(A_RMODE);
  Keepmode=EEPROM.read(A_KEEPMODE);

  Smode=EEPROM.read(A_SMODE);
  if (Smode>3 || Smode <1) 
  {
	  Smode=1;
	  EEPROM.update(A_SMODE,Smode);
	  EEPROM.update(A_KEEPMODE,false);
  } 
  else if (Keepmode) EEPROM.update(A_KEEPMODE,false);
  else if (Smode==2) Rmode=false;  //config
  else if (Smode==3) Rmode=true;   //runmode
  
  for (byte n=1; n<MAXTERM; n++)  //Read all settings from memory
  {
    AllVar[n].DelWait=0;
    LastInput[n]=HIGH;
	AllVar[n].InpWait=0;
	
	EEPROM.get(A_PORT+n*A_GETPUT,AllPorts[n]);

    AllVar[n].free=true;
    if (AllPorts[n].address==65535) AllPorts[n].address=0;
    if (AllPorts[n].type==T_Input) //input
    {
      pinMode(p[n],INPUT_PULLUP); 
      AllVar[n].output=1;
      AllVar[n].swrelay=1;
    } else pinMode(p[n],OUTPUT);
/*
    if (AllPorts[n].type==T_Access) 
	{
		AllVar[n].output=EEPROM.read(A_OUTSET+n);
		AllVar[n].swrelay=AllVar[n].output;
	}
*/
    if (AllPorts[n].type==T_Servo) 
    {
      if (AllPorts[n].SpeedMode==Val255) AllPorts[n].SpeedMode=defspeed;
      AllVar[n].whoek=(AllPorts[n].Hangle+AllPorts[n].Langle)/2;
      Servos[AllPorts[n].RiseOn].detach();
      if (AllPorts[n].frog2<MAXTERM) digitalWrite(p[AllPorts[n].frog2],AllPorts[n].Invert);
    } 
  }  
}
//==================================================================
void StartScreen()
{
  Serial.begin(57600);
  delay(100);
  Serial.println(F("\e[2J\e[1;1H\e[33m\e[1mMARDEC, the Multifunctional ARduino dcc DECoder"));
  if (MEGA) Serial.print(F("MEGA")); else Serial.print(F("UNO"));
 // Serial.print(versie); 
  Serial.println(" version 6.0b");
  Serial.println(F("\nMardec starting, please wait"));

//  Serial.println(sizeof(PortItem));
/*  
  if (EEPROM.read(MAXMEM-1)!=resetval) 
  {
      for (int i=0;i<MAXMEM;i++) EEPROM.update(i,Val255);    
      for (byte n=1; n<MAXTERM; n++) EEPROM.update(A_OUTSET+n,0);      
      EEPROM.update(MAXMEM-1,resetval);
  }
*/
}
//-------------------------------------------------
void SetupCon()
{
  dwit();
  Serial.print(F("\nConfiguration mode of MARDEC #"));
  Serial.println(decnum);
  dgeel();
  changes=false;
  pinsel=false;
  Document();
  digitalWrite (LED_BUILTIN,HIGH);
}
//---------------------------------------------
void SetupNormal()
{
  dwit();
  Serial.print(F("\nNormal mode of MARDEC #"));
  Serial.println(decnum);
  dgeel();
  Serial.println(F("To configuration: connect USB and press 'C' or 'P'\n")); 
//  Serial.println("");
  pinMode(2,INPUT_PULLUP); //DCC input
  DCC.SetBasicAccessoryDecoderPacketHandler(BasicAccDecoderPacket_Handler, true);
  DCC.SetupDecoder (0x00, 0x00, kDCC_INTERRUPT);
}

//--------------------------------
void BasicAccDecoderPacket_Handler(int address, boolean activate, byte data)
{
  
  // Convert NMRA packet address format to human address
  address -= 1;
  address *= 4;
  address += offset;
  address += (data & 0x06) >> 1;

  bool enable = (data & 0x01) ? 1 : 0;
//   Serial.println(address);
  for (int i=1; i<MAXTERM; i++)
  {
    if (address == AllPorts[i].address)
    {
//	if ((AllPorts[i].type==T_Access  || AllPorts[i].type==T_Servo) || 
//	    (AllPorts[i].type==T_Input && AllPorts[i].frogbud=='B'))
	if (AllPorts[i].type==T_Access  || AllPorts[i].type==T_Servo) 
      {
        AllVar[i].output=enable;
        LED13state=true;
        digitalWrite (LED_BUILTIN,LED13state);
        l13mil=millis(); 
      }
    }
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void loop() 
{
    if (Rmode) NormalMode(); else ConfigureMode();
}
//====================================
void ConfigureMode()
{
  Serial.println("");
  do  
  {
    gencmd=EEPROM.read(MAXMEM);
    if (gencmd=='P') EEPROM.update(MAXMEM,0); else {Serial.print(F("\nEnter command (P/A/D/E/I/R/?): ")); gencmd=GetCommand();}
	if (gencmd=='X') gencmd='?';
    if (gencmd=='R') ResetAll();
	else if (gencmd=='I') AskNum21();
    else if (gencmd=='D') Document();
    else if (gencmd=='?') Serial.println(F("\nP Select port by number\n\rA Select port by Address\n\r"
                       "D Display all\n\rE Exit to normal mode\n\r"
		               "I set basic Information\n\rR Reset MARDEC"));
    else if (gencmd=='P') ProcessPort(0);
	else if (gencmd=='A') AdresPort();
	else if (gencmd!='E') ShowInvalid();
  } 
  while (gencmd != 'E');
  
  ExitNormal();
} 
//-----------------------------
 void ExitNormal()
{
  EEPROM.update(MAXMEM,0); 
  EEPROM.update(A_RMODE,true);  //Force normal mode
  EEPROM.update(A_KEEPMODE,true);
  delay(1000);
  asm volatile ("  jmp 0");  //herstart 
}
//---------------------
void AdresPort()
{
int padres;
byte p;
	Serial.println(F("Enter DCC address"));
	padres=AskInteger(0,1,MAXDCC);
	if (padres==0) return;
	for (p=1;p<MAXTERM;p++) if (AllPorts[p].address==padres) 
	{
		ProcessPort(p);
		return;
	}
	if (p==MAXTERM) {dblauw(); Serial.println(F("\nNo such address")); dgeel();}
}
//---------------------
void dwit()
{
  Serial.print("\e[37m\e[1m");
}
//---------------------
void dgeel()
{
  Serial.print("\e[33m\e[1m");
}
//---------------------
void dblauw()
{
  Serial.print(char(7));
  Serial.print("\e[36m\e[1m");
}
//---------------------------- ProcessPort
void ProcessPort(byte pin)
{
  if (pin==0) curport=AskPortNumber(); else curport=pin;
  if (curport==-1) return;
  pinsel=true;
  if (AllPorts[curport].type==Val255) //undefined pin
  {
    Serial.println(F("\nPin is undefined. Enter DCC address (0=cancel)"));
    AskDCC(0);   
    if (AllPorts[curport].address==0) {pinsel=false; return;}	
    PortType();
    if (AllPorts[curport].type==T_Access) 
    {
		ResetPort(false);	
//	  AllPorts[curport].RiseOn=0;
//	  AllPorts[curport].FallOffBH=0;
//	  AllPorts[curport].BounceLOW=0;
//	  AllPorts[curport].Langle=0;
//	  AllPorts[curport].Hangle=0;
//	  AllPorts[curport].SpeedMode=0;
      AskAccMode();
//	  AllPorts[curport].Invert=false;
//	  AllPorts[curport].admin[0]='\0';
	  AllVar[curport].output=0;
	  EEPROM.update(A_OUTSET+curport,0);  
      AccessoryPort();
    }
    else if (AllPorts[curport].type==T_Servo) 
    {   
      NewServo();
      ServoPort();
    }
    else if (AllPorts[curport].type==T_Input) //input
    {
 	  AllPorts[curport].admin[0]='\0';
      AllPorts[curport].Invert=0;
	  AllPorts[curport].frogbud=GetUDB(); //AskIType();
	  InputPort();
    }
  }
  else //already defined pin --> changes
  {
    if (AllPorts[curport].type==T_Access)    // Existing accessory
    {
      Serial.println("\e[37m\e[1m");
      ShowDCC(curport); ShowAccessory(curport);
      Serial.println("\e[33m\e[1m");
	  AllVar[curport].output=0;
      AccessoryPort();
    }
    else if (AllPorts[curport].type==T_Servo)      // Existing servo  @@@@@@@@
    {
      Servos[AllPorts[curport].RiseOn].attach(p[curport]);
      if (AllPorts[curport].frog2<MAXTERM) digitalWrite(p[AllPorts[curport].frog2],!AllPorts[AllPorts[curport].frog2].Invert); // koppel puntstuk
	  AllVar[curport].angle=AllVar[curport].setpoint;
	  Servos[AllPorts[curport].RiseOn].write(AllVar[curport].angle);
      Serial.println("\e[37m\e[1m");
      ShowDCC(curport); 
	  ShowServo(curport);
      Serial.println("");
//      ShowAngle(curport);
	  
	  if (AllVar[curport].angle==AllPorts[curport].Langle) cmdLH[curport] = 'L'; else cmdLH[curport] = 'H';
	  AllVar[curport].setpoint=AllVar[curport].angle;
	  ShowHoek(curport);
	  
      ServoPort();
    }
    else if (AllPorts[curport].type==T_Input) 
    {
      Serial.println("\e[37m\e[1m");
      ShowDCC(curport); ShowInput(curport);
//      Serial.println("");
      Serial.println("\e[33m\e[1m");
      InputPort();
    }
  }
  if (AllPorts[curport].type==T_Servo) Servos[AllPorts[curport].RiseOn].detach();
  if (changes) SavePort();
}
//------------------------------ InputPort
void InputPort()
{
  do
  {
    Serial.print("\nEnter command for Input");
//    Serial.println(curport);
	PortAdres(curport,true);
    Serial.print(F("T/2/3/enter/R/I/A/N/D/E/? : "));
    gencmd=GetCommand();
    if (gencmd=='N') GetAdmNote();
    else if (gencmd=='A') AskDCC(1);
    else if (gencmd=='?') 
	{
		Serial.print(F("\n\rT trigger Type\n\r2 Second address\n\r3 Third address"));   
		ShowDAN();
	}
    else if (gencmd=='D') Document();
	else if (gencmd=='I') AskInvert();
	else if (gencmd=='R') ClearPort();
	else if (gencmd=='E') {SavePort(); ExitNormal();} 
    else if (gencmd=='2') AskDCC2();
    else if (gencmd=='3') AskDCC3();
	else if (gencmd=='T') AllPorts[curport].frogbud=GetUDB();
	else if (gencmd!='X') ShowInvalid();
  } 
  while (gencmd != 'X');
}
//----------------------------
void ShowDAN()
{
	Serial.println(F("\n\rI Invert pin\n\rA DCC-Address\n\r"
	              "D Display all\n\rN admin. Note\n\rR Reset pin"
				  "\n\rEnter/X Save pin\n\rE Save pin and start Normal mode"));
}
//-----------------------------
void PortAdres (byte pin, boolean NL)
{
	Serial.print(" on A");
	Serial.print(AllPorts[pin].address);
	Serial.print("/P");
	Serial.print(p[pin]);
	if (NL) Serial.println(""); 
}
//-----------------------------
void AskDCC2()
{
int dcc2;
//  dcc2=AllPorts[curport].RiseOn;
  dcc2=AskDCC23(AllPorts[curport].RiseOn,(char *)"Second");
  if (dcc2==0)
  {
    AllPorts[curport].RiseOn=65535;
    AllPorts[curport].Langle=Val255;
	changes=true;
    return;
  }
  else AllPorts[curport].RiseOn=dcc2;
  
  changes=true;
  if (AllPorts[curport].Langle==Val255) AllPorts[curport].Langle=0;
  AllPorts[curport].Langle=DelayInp(AllPorts[curport].Langle);  //vraag delay
  Show23(dcc2,AllPorts[curport].Langle,(char *)"Second");
}
//----------------------------- AskDCC23
int AskDCC23(int dcc23, char *tekst)
{
int d23;
  Serial.print(F("Enter "));
  Serial.print(tekst);
  Serial.print(F(" control address."));
  if (dcc23<MAXDCC && dcc23>0)
  {
    Serial.println (F("0=remove"));
    d23=AskInteger(dcc23,0,MAXDCC);
  } 
  else 
  {
    Serial.println("");
    d23=AskInteger(0,1,MAXDCC);
  }
  return d23;
}
//----------------------------- Show23
void Show23(int dcc, int del, char *tekst)
{
  dwit();
  Serial.print(F("\nAddress "));
  Serial.print(dcc);
  Serial.print(F(" set as "));
  Serial.print(tekst);
//  Serial.println(F(" control address."));
//  Serial.print(F("With a delay of "));
  Serial.print(" control address with a delay of ");
  Serial.print(del/10);
  Serial.print(".");
  Serial.print(del%10);
  Serial.println(F(" sec."));
  dgeel(); 
}
//----------------------------- DelayInp
byte DelayInp(byte tijd)
{
byte later;
  Serial.println(F("\nSet delay time in 0.1 seconds."));
  later=AskInteger(tijd,0,250);
  return later;
}
//-----------------------------
void AskDCC3()
{
int dcc3;
//  dcc3=AllPorts[curport].FallOffBH;
  dcc3=AskDCC23(AllPorts[curport].FallOffBH,(char *)"Third");
  if (dcc3==0)
  {
    AllPorts[curport].FallOffBH=65535;
    AllPorts[curport].Hangle=Val255;
    changes=true;
    return;
  }
  else AllPorts[curport].FallOffBH=dcc3;
  
  changes=true;
  if (AllPorts[curport].Hangle==Val255) AllPorts[curport].Hangle=0;
  AllPorts[curport].Hangle=DelayInp(AllPorts[curport].Hangle);  //vraag delay
  Show23(dcc3,AllPorts[curport].Hangle,(char *)"Third");
}
//------------------------------ ;ShowAngle
/*
void ShowAngle(byte n)
{
  dwit();
  if (AllVar[curport].angle==AllPorts[curport].Langle) 
  {
    Serial.print(F("Angle set to LOW: "));
    cmdLH[curport] = 'L';
  }
  else //if (AllPorts[curport].angle==AllPorts[curport].Hangle) 
  {
    Serial.print(F("Angle set to HIGH: "));
    cmdLH[curport] = 'H';
  }
  Serial.println(AllVar[curport].angle);
  dgeel();
}
*/
//--------------------------- MoveSpeed
int MoveSpeed()
{
byte curval;
    curval=AllPorts[curport].SpeedMode;
    Serial.print(F("\nSet speed for servo"));
    PortAdres(curport,true);
    AllPorts[curport].SpeedMode = AskInteger(curval,5,MAXSPEED);
    dwit();
    Serial.print(F("\nSpeed set to "));
    Serial.print(AllPorts[curport].SpeedMode); 
    Serial.println(F(" ms/step")); 
    dgeel();  
    if (AllPorts[curport].SpeedMode != curval) changes=true;
}
//============================== DefaultSpeed
void DefaultSpeed()
{
int curval;
    curval=defspeed;
    Serial.println(F("\nDefault rotation speed for all servo's"));
    defspeed = AskInteger(curval,5,MAXSPEED);
    dwit();
    Serial.print(F("\nDefault speed set to "));
    Serial.print(defspeed); 
    Serial.println(F(" ms/degree")); 
    dgeel();  
    EEPROM.update(A_DEFSPEED,defspeed);
    for (byte n=1;n<MAXTERM;n++) if (AllPorts[n].SpeedMode==curval && AllPorts[n].type==T_Servo) AllPorts[n].SpeedMode=defspeed;
}
//---------------------------- porttype
void PortType()
{
byte curval=0;
  do
  {
   Serial.print(F("\nSelect pin type(enter=cancel)\n\rAccessory(A), Input(I)"));
   if (numservo==MAXSERVO) 
       Serial.print(F(":"));   //all servo's used
    else
      Serial.print(F(" or Servo(S): "));
	gencmd=GetCommand();
	if (gencmd=='X') {changes=false; AllPorts[curport].address=0; return;}
	else if (gencmd=='A') curval=1;
	else if (gencmd=='S') curval=2;
	else if (gencmd=='I') curval=3;
	else ShowInvalid();
	if (curval==2 && numservo==MAXSERVO) curval=0;
  } while (curval==0);
  dwit();
  Serial.print(F("\nPin "));
  Serial.print(p[curport]);
  if (curval==T_Access) Serial.println(F(" set as Accessory port."));
  else if (curval==T_Servo) 
  {
    Serial.println(F(" set as Servo port."));
//	AllPorts[curport].RiseOn=FindFreeServo();
//    numservo++;
  }
  if (curval==3) 
  {
    Serial.println(F(" set as Input port."));
    curval=T_Input;
  }
  dgeel();
  AllPorts[curport].type = curval;
}
//----------------------------
byte FindFreeServo()
{
	for (byte i=0;i<12;i++) if (SerUsed[i]==0)
	{
		SerUsed[i]=1;
//		Serial.println(i);
		return i;
	}
}
//----------------------------- SavePort 
void SavePort()
{
	EEPROM.put(A_PORT+(curport*A_GETPUT),AllPorts[curport]);

    if (AllPorts[curport].frogbud < MAXTERM)  //           FP/buddy only
    {   
      EEPROM.put(A_PORT+(A_GETPUT*AllPorts[curport].frogbud),AllPorts[AllPorts[curport].frogbud]);  //Buddy of FP1
	}
    if (AllPorts[curport].frog2<MAXTERM)  //tweede FP
    {
      EEPROM.put(A_PORT+(A_GETPUT*AllPorts[curport].frog2),AllPorts[AllPorts[curport].frog2]);  //FP2
    }

	if (AllPorts[curport].type==T_Access) //directe aanpassing van inversie van accessory
	{
		digitalWrite (p[curport],AllPorts[curport].Invert^LOW);
		if (AllPorts[curport].frogbud<MAXTERM) digitalWrite (p[AllPorts[curport].frogbud],AllPorts[curport].Invert^LOW);
	}
//	EEPROM.update(A_OUTSET+curport,AllVar[curport].output);	
	EEPROM.update(A_OUTSET+curport,AllVar[curport].setpoint);
    changes=false;
    dwit();
    Serial.println(F("\nPin settings are saved!"));
    dgeel();
}
  //---------------------------- GetCommand
char GetCommand()
{
byte Command;

  FlushWait();
  Command=Serial.read();
  if (Command==13) // Just enter
  {
	pinsel=false;
	Command='X';
	valid=true;
  }
  else if (Command>95) Command=Command-32;
  Serial.println ("");

  return Command;
}
//---------------------------- ShowInvalid
void ShowInvalid()
{
	dblauw();
	Serial.println("\rCommand or option invalid or not applicable");
	dgeel();
}

//--------------------------- NewServo
void NewServo()
{
	  AllPorts[curport].Invert = 0;
	  AllPorts[curport].Langle=75;
	  AllPorts[curport].Hangle=105;
	  AllVar[curport].whoek=90;
	  AllPorts[curport].BounceLOW=0;
	  AllPorts[curport].FallOffBH=0;
	  AllPorts[curport].admin[0]='\0';
	  AllPorts[curport].SpeedMode=defspeed;
	  AllPorts[curport].RiseOn=FindFreeServo();
	  numservo++;
	  Servos[AllPorts[curport].RiseOn].attach(p[curport]);
	  AllVar[curport].output=1;
	  AllVar[curport].angle=90; //force to centerposition to prevent sweep to 0
	  SetAngleN(curport);
	  for (byte i=0;i<4;i++) {BounceHL[AllPorts[curport].RiseOn][i][0]=0; BounceHL[curport][i][1]=0;}
}
//---------------------------- ResetAll
void ResetAll()
{
	Serial.print(F("\nClear all settings? Y/N [N]: "));
	if (AskYN()=='Y')
	{
		Serial.print(F("ABSOLUTELY sure? Y/N [N]: "));
		if (AskYN()=='Y')
		{
			for (int n=0;n<MAXMEM;n++) EEPROM.update(n,Val255);
			for (byte b=0;b<MAXTERM;b++) digitalWrite (b,LOW);
			EEPROM.update(A_SMODE,1);
			EEPROM.update(A_DECNUM,Val255);
			EEPROM.update(A_DEFSPEED,25);
			EEPROM.update(A_DETACH,true);
			EEPROM.update(A_OFFSET,1);
			EEPROM.update(A_RMODE,false);
			EEPROM.update(A_KEEPMODE,true);
		//      ;
			asm volatile ("  jmp 0");  //herstart 
		}
	}
}
//---------------------------- Clear Port
void ClearPort()
{
	Serial.print(F("\nDo you want to reset pin "));
	Serial.print(p[curport]);
	Serial.print(F("? Y/N [N]: "));
	if (AskYN() == 'Y')
	{  
		if (AllPorts[curport].type==T_Servo) 
		{
			SerUsed[AllPorts[curport].RiseOn]=0;
			numservo--;
		}
		ResetPort(true);
		dwit();
		Serial.print(F("pin "));
		Serial.print(p[curport]);
		Serial.println(F(" is now reset"));
		dgeel();
		pinsel=false; 
		SavePort();	
		gencmd='X';
	}
}
//---------------------------- resetPort
void ResetPort(boolean Alles)
{
	AllPorts[curport].Langle=0;
    AllPorts[curport].Hangle=Val255;
	AllPorts[curport].admin[0]='\0'; 
	AllPorts[curport].Invert=false;  
	AllPorts[curport].RiseOn=0; 
	AllPorts[curport].FallOffBH=0; 		
    AllPorts[curport].SpeedMode=Val255;
	if (Alles)
	{
		if (AllPorts[curport].frogbud<MAXTERM) ClearFrogBud(AllPorts[curport].frogbud);
		if (AllPorts[curport].frog2<MAXTERM) ClearFrogBud(AllPorts[curport].frog2);
		AllPorts[curport].type=Val255;
		AllPorts[curport].address=0;  	
	}
}
//---------------------------- Document
void Document()
{
  Serial.print(F("\nSettings of MARDEC #"));
  Serial.println(decnum);
  Serial.print(F("\nDefault servo rotation speed: "));
  Serial.print(defspeed);
  Serial.println(F(" ms/degree"));
  Serial.print(F("Address offset: "));
  if (offset==1) Serial.println(F("No")); else Serial.println(F("Yes"));
  Showdeta();
  ShowSM();
  
  for (byte n=1;n<MAXTERM;n++)
  {
//	  Serial.println(AllPorts[n].type);
      if (p[n]<10) Serial.print(F("pin  ")); else Serial.print(F("pin "));
      // Added pin number
      //Serial.print(n); //Serial.print(" (pin "); 
	  Serial.print(p[n]);
      Serial.print(": "); 
      if (AllPorts[n].type==Val255) Serial.print(F("not configured"));
      else if (AllPorts[n].type==T_Access) {ShowDCC(n); ShowAccessory(n);}
      else if (AllPorts[n].type==T_Servo) {ShowDCC(n); ShowServo(n);}   
      else if (AllPorts[n].type==T_Input) {ShowDCC(n); ShowInput(n);}   
      if (AllPorts[n].type>T_Servo && AllPorts[n].type<T_Input) ShowBudFrog(n);    
      Serial.println("");
  }
  Serial.println("\n");
 }
//---------------------------- ShowDCC
void ShowDCC(byte n)
{
  Serial.print(F("DCC "));
  Serial.print(AllPorts[n].address);
}
//---------------------------- ShowInput
void ShowInput(byte n)
{
int a23;  
  Serial.print(F(", Input "));
  Serial.print(AllPorts[n].admin);
  Serial.print((AllPorts[n].Invert) ? F(", Inverted") : F(", Not Inv."));
  Serial.print(F(", Trigger: "));
  Serial.print(char(AllPorts[n].frogbud));
  Serial.print(F(", Second addr/delay: "));
  a23=AllPorts[n].RiseOn;
  if (a23<MAXDCC && a23>0) shdel(a23,AllPorts[n].Langle); else Serial.print(F("none"));
  Serial.print(F(", Third addr/delay: "));
  a23=AllPorts[n].FallOffBH;
  if (a23<MAXDCC && a23>0) shdel(a23,AllPorts[n].Hangle); else Serial.print(F("none"));
}
void shdel(int adres, int tijd)
{
    Serial.print(adres);
    Serial.print("/"); 
    Serial.print(tijd/10);
    Serial.print(".");
    Serial.print(tijd%10);
    Serial.print(F(" sec"));  
}
//---------------------------- ShowBudFrog 
void ShowBudFrog(byte n)
{
  if (AllPorts[n].type==T_Frog1) 
  {
    Serial.print(F("First relay (")); 
    if (AllPorts[n].Invert) Serial.print(F("Inv.)")); else Serial.print(F("No inv.)"));
  }
  else if (AllPorts[n].type==T_Frog2) 
  {
    Serial.print(F("Second relay (")); 
    if (AllPorts[n].Invert) Serial.print(F("Inv.)")); else Serial.print(F("No inv.)"));
  }
  else if (AllPorts[n].type==T_Buddy) Serial.print(F("Buddy"));
  Serial.print(F(" for pin "));
  Serial.print(p[AllPorts[n].frogbud]);
}
//---------------------------- ShowServo
void ShowServo(byte n)
{
  Serial.print(F(", Servo "));
  Serial.print(AllPorts[n].admin);
  Serial.print((AllPorts[n].Invert) ? F(", Inverted") : F(", Not Inv."));
  Serial.print(F(", Angles "));
  Serial.print(AllPorts[n].Langle);
  Serial.print(F("/"));
  Serial.print(AllPorts[n].Hangle);
  
  Serial.print(F(", Speed ")); 
  Serial.print(AllPorts[n].SpeedMode);
  
  Serial.print(F(", Frog pin "));
  if (AllPorts[n].frogbud<MAXTERM) 
  {
   Serial.print(p[AllPorts[n].frogbud]);
   if (AllPorts[n].frog2<MAXTERM) 
   {
    Serial.print(F("/"));
    Serial.print(p[AllPorts[n].frog2]);
   }
  }
  else Serial.print(F("no"));
  
  Serial.print(F(", Bounce (L/H): "));
  if (AllPorts[n].BounceLOW<10000) Serial.print(AllPorts[n].BounceLOW); else Serial.print(0);
  Serial.print(F("/"));
  if (AllPorts[n].FallOffBH<10000) Serial.print(AllPorts[n].FallOffBH); else Serial.print(0);
//  Serial.print(F(" "));
//  Serial.print(AllPorts[n].RiseOn);
}
//---------------------------- ShowAccessory
void ShowAccessory(byte n)
{
  Serial.print(F(", Acc.type ")); 
  Serial.print(AllPorts[n].SpeedMode);
  Serial.print(F(" (")); 
  if (AllPorts[n].SpeedMode>0 and AllPorts[n].SpeedMode<=6) Serial.print( (AllPorts[n].SpeedMode & 1) ? F("S. "): F("D. "));
  switch (AllPorts[n].SpeedMode)
  {
    case M_SiSteady  :                               // same as case 2
    case M_DuSteady  :Serial.print(F("Steady"));     	break;
    case M_SiFlash   :                               // same as case 4
    case M_DuFlash   :Serial.print(F("Flashing"));   	break;
    case M_SiOneShot :                               // same as case 6
    case M_DuOneShot :Serial.print(F("One shot"));  	break;
    case M_AnalPWM   :Serial.print(F("PWM"));        	break;
    case M_FirePWM   :Serial.print(F("Flicker"));    	break;
    case M_Random  :Serial.print(F("Random"));  break;
 //   case M_RandShort :Serial.print(F("Random Short")); break;
  }

  Serial.print(F("), "));
  Serial.print(AllPorts[n].admin);
  Serial.print((AllPorts[n].Invert) ? F(", Inv.") : F(", Not Inv.")); 

  if (AllPorts[n].SpeedMode==M_DuSteady || AllPorts[n].SpeedMode==4 || AllPorts[n].SpeedMode==6)
  {
    Serial.print(F(", Buddy "));      // Do_246
    if (AllPorts[n].frogbud<MAXTERM)
         Serial.print(p[AllPorts[n].frogbud]);
    else Serial.print(F("no "));
  }
  
  if (AllPorts[n].SpeedMode>=M_SiFlash && AllPorts[n].SpeedMode<=M_DuOneShot) //flashing en one shot (milliseconden)  3 4 5 6
  {
    Serial.print(F(", Time(ms) ")); 		   //Do_3456
    Serial.print(AllPorts[n].RiseOn);
    if (AllPorts[n].SpeedMode<M_SiOneShot)     // 3 en 4	Do_34
    {
      Serial.print(F("/"));
      Serial.print(AllPorts[n].FallOffBH);
    }
	if (AllPorts[n].SpeedMode==M_SiOneShot)
	{
		Serial.print(F(", Trigger: ")); 		//Do_5
		Serial.print(char(AllPorts[n].frog2));
	}
  }
  
  else if (AllPorts[n].SpeedMode==M_AnalPWM  || AllPorts[n].SpeedMode==M_FirePWM)    //Pwm level
  {
    Serial.print(F(", PWM ")); 		//Do_78
    Serial.print(AllPorts[n].Langle);     
    Serial.print(F("/"));
    Serial.print(AllPorts[n].Hangle);
  }   
  if (AllPorts[n].SpeedMode==M_Random || AllPorts[n].SpeedMode==M_AnalPWM)   //do_79
  {  
    if(false) {  }
    else if (AllPorts[n].SpeedMode==M_Random) 
	{
		Serial.print(F(", Time(sec) ")); //Do_9
		Serial.print(AllPorts[n].RiseOn*0.02);          
		Serial.print(F("/"));
		Serial.print(AllPorts[n].FallOffBH*0.02);  
		if (AllPorts[n].frog2==2) Serial.print(F(" FixOn "));
		if (AllPorts[n].frog2==3) Serial.print(F(" FixOff "));
		if (AllPorts[n].frog2>1)  Serial.print(AllPorts[n].BounceLOW*0.02);  
	}
    else 
	{
		Serial.print(F(", Time R/F(sec) ")); //Do_7
		Serial.print(AllPorts[n].RiseOn);          
		Serial.print(F("/"));
		Serial.print(AllPorts[n].FallOffBH);  
	}
  }
}
//--------------------------- DCCval
void DCCval(byte val,boolean st)
{
	if (st) Serial.println(F("Press 'E' or 'Enter' to stop testing or 'T' for on/off"));
	Serial.print("DCC value: ");
	Serial.println(val);
}
//--------------------------- TestAccessory
void TestAccessory(byte n)
{
char ST;
	AllVar[n].swrelay=0;
	AllVar[n].output=1;
	AllVar[n].free=true;
  
	DCCval(AllVar[n].output,true);
	do
	{
	  switch (AllPorts[n].SpeedMode)
	  {
		case M_SiSteady  :                               // same as case 2
		case M_DuSteady  : ProcesSteady(n);    	break;
		case M_SiFlash   :                               // same as case 4
		case M_DuFlash   : ProcesFlash(n);    	break;
		case M_SiOneShot :                               // same as case 6
		case M_DuOneShot : ProcesOneShot(n);  	break;
		case M_AnalPWM   : ProcessPWM(n);       break;
		case M_FirePWM   : ProcessFire(n);    	break;
		case M_Random    : ProcesRandom(n);     break;
	  }
	  ST=TestWait(n);
	}	
	while (ST!='S' && ST!='E');
	EEPROM.update(A_OUTSET+n,AllVar[n].output);	
    AllOff();  
  if (ST=='E') {SavePort(); ExitNormal();} 
}

//----------------
void AllOff()
{
	for (byte i=0;i<MAXTERM;i++) if (AllPorts[i].type==T_Access)
	{
		digitalWrite(p[i],LOW);
		if (AllPorts[i].frogbud<MAXTERM) digitalWrite(p[AllPorts[i].frogbud],LOW);
	}
}
//---------------------------- AskPortNumber
byte AskPortNumber()
{
byte cpin; 
byte ppin;
  valid = false;
  while (!valid)
  {
    Serial.println(F("\n\rSelect pin number."));
    ppin = AskInteger(0,3,19);
	if (ppin>12) cpin=ppin-3; else cpin=ppin-2;
    if (AllPorts[cpin].type==T_Buddy || AllPorts[cpin].type==T_Frog1 || AllPorts[cpin].type==T_Frog2)
//	if (AllPorts[cpin].type!=Val255)
    {
        dblauw();
        Serial.print(F("\nPin "));
        Serial.print(p[cpin]);
        Serial.println(F(" is buddy/frog pin "));
        dgeel();
      } else if (ppin!=13) valid=true;
   } 
  return cpin;
}
//---------------------------- ServoPort
void ServoPort()
{
  do
  {
    Serial.print(F("\nEnter command for Servo"));
	PortAdres(curport,true);
    Serial.print(F("+/-/9/C/F/I/B/S/T/R/A/D/N/enter/E/?: "));
//    Serial.print(F("+/-/9/C/F/I/B/S/T/A/D/N/enter/?: "));
    gencmd=GetCommand();
	if (gencmd=='N') GetAdmNote();
	else if (gencmd=='9') SetAngle90(); 
	else if (gencmd=='+') CountUp(); 
	else if (gencmd=='-') CountDown(); 
	else if (gencmd=='C') ChangeAngle(); 
	else if (gencmd=='T') SetServo();
	else if (gencmd=='I') AskInvert(); 
	else if (gencmd=='F') AskFrog(); 
	else if (gencmd=='R') ClearPort(); 
	else if (gencmd=='D') Document(); 
	else if (gencmd=='S') MoveSpeed();
	else if (gencmd=='A') AskDCC(1); 
	else if (gencmd=='E') {SavePort(); ExitNormal();} 
	else if (gencmd=='B') Bouncing();
	else if (gencmd=='?')  
	{
		Serial.print(F("\n\r+ raise angle\n\r- lower angle\n\r"
				   "9 90 degr.\n\rC Low<->High\n\r"
				   "F Frog port\n\rB Bouncing"
				   "\n\rS set Speed\n\rT Test Servo"));
		ShowDAN();
	}
	else if (gencmd!='X') ShowInvalid();
  } 
  while (gencmd != 'X');
}
//============================== 
void SetServo()
{
	if (cmdLH[curport] == 'H') {AllVar[curport].angle = AllPorts[curport].Langle; cmdLH[curport]='L';}
	else                       {AllVar[curport].angle = AllPorts[curport].Hangle; cmdLH[curport]='H';}
	ShowHL();
	if (cmdLH[curport]=='L') AllVar[curport].output=AllPorts[curport].Invert;
	else AllVar[curport].output=1-AllPorts[curport].Invert;
	Servos[AllPorts[curport].RiseOn].write(AllVar[curport].angle);
	DCCval(AllVar[curport].output,false);
}
//=====================================
void Bouncing()
{
int nr;
	Serial.println(F("Set bounce angles for Low angle"));
	nr	= AskInteger(AllPorts[curport].BounceLOW,0,9999);
	if (AllPorts[curport].BounceLOW != nr) {changes=true; AllPorts[curport].BounceLOW=nr;}
	Serial.println(F("\nSet bounce angles for High angle"));
	nr = AskInteger(AllPorts[curport].FallOffBH,0,9999);
	if (AllPorts[curport].FallOffBH != nr) {changes=true; AllPorts[curport].FallOffBH=nr;}	
	dwit();
    Serial.print(F("\nBounce angles set to (L-H) "));
    Serial.print(AllPorts[curport].BounceLOW);
	Serial.print(F("-"));
	Serial.print(AllPorts[curport].FallOffBH);
	Serial.println("\e[33m\e[1m");
	SetBounce(curport);
}
//----------------------------------------- 
void SetBounce(byte port)
{
	if (AllPorts[port].BounceLOW>0) CvtBounce(AllPorts[port].BounceLOW,0,port);
	if (AllPorts[port].FallOffBH>0) CvtBounce(AllPorts[port].FallOffBH,1,port);
}
//-------------------------------- AccessoryPort
void AccessoryPort()
{
  do
  {
    Serial.print(F("\nEnter command for Accessory"));
 	PortAdres(curport,true);
    Serial.print(F("M/I/R/T/A/D/N/E/enter/?: "));
	gencmd=GetCommand();
    if (gencmd=='N') GetAdmNote();
    else if (gencmd=='M') AskAccMode();
    else if (gencmd=='R') ClearPort();
//    else if (gencmd=='H' && AllPorts[curport].SpeedMode>=M_SiFlash  && AllPorts[curport].SpeedMode<=M_DuOneShot)  // 3 4 5 6 Flashing d/o shot
//       AllPorts[curport].RiseOn=AskOnOffTime((char *)"On",AllPorts[curport].RiseOn);
//    else if(gencmd=='L' && (AllPorts[curport].SpeedMode==M_SiFlash || AllPorts[curport].SpeedMode==M_DuFlash)) // 3 4 Flashing
//       AllPorts[curport].FallOffBH=AskOnOffTime((char *)"Off",AllPorts[curport].FallOffBH);
//    else if (gencmd=='H' && (AllPorts[curport].SpeedMode==M_AnalPWM ||AllPorts[curport].SpeedMode==M_FirePWM)) // pwm 7 8 
//      AllPorts[curport].Hangle=AskPWM((char *)"Maximum",AllPorts[curport].Hangle);
//    else if (gencmd=='L' && (AllPorts[curport].SpeedMode==M_AnalPWM ||AllPorts[curport].SpeedMode==M_FirePWM)) //pwm 7 8
//      AllPorts[curport].Langle=AskPWM((char *)"Minimum",AllPorts[curport].Langle);
//    else if(gencmd=='H' && AllPorts[curport].SpeedMode>=M_Random)    // 9 10
//       AllPorts[curport].FallOffBH=AskRandom((char *)"Maximum",AllPorts[curport].FallOffBH);
//    else if (gencmd=='L' && AllPorts[curport].SpeedMode>=M_Random)  // 9 10
//       AllPorts[curport].RiseOn=AskRandom((char *)"Minimum",AllPorts[curport].RiseOn);
//    else if (gencmd=='F' && AllPorts[curport].SpeedMode==M_AnalPWM) //7
//       AllPorts[curport].FallOffBH=AskRiseFall((char *)"Fall",AllPorts[curport].FallOffBH);
//	else if (gencmd=='F' && AllPorts[curport].SpeedMode==M_Random) AskFix(); // 9  Fix
//    else if (gencmd=='R' && AllPorts[curport].SpeedMode==M_AnalPWM) //7
//       AllPorts[curport].RiseOn=AskRiseFall((char *)"Rise",AllPorts[curport].RiseOn);
    else if (gencmd=='D') Document();
    else if (gencmd=='T') TestAccessory(curport);      
    else if (gencmd=='A') AskDCC(1);
	else if (gencmd=='E') {SavePort(); ExitNormal();} 
    else if (gencmd=='?') 
		{
			Serial.print(F("\n\rT Test pin"
					 "\n\rM Modify mode"));
			ShowDAN();
		}
    else if (gencmd=='I') AskInvert();
/*    {
      AllPorts[curport].Invert=1-AllPorts[curport].Invert;
	  changes=true;
      dwit();
	  Serial.print(F("\nAccessory"));
      PortAdres(curport,false);
      if (AllPorts[curport].Invert) Serial.println (F(" set to 'Inverted'")); else Serial.println (F(" set to 'NOT Inverted'")); 
      dgeel();
    }
    else if (gencmd != 'X') ShowInvalid();
*/
  }  
  while (gencmd != 'X');
}
//================================== ChangeAngle
void ChangeAngle()
{
//  DCCval(AllVar[curport].output,false);
  if (cmdLH[curport]=='L') AllVar[curport].output=AllPorts[curport].Invert;
  if (cmdLH[curport]=='H') AllVar[curport].output=1-AllPorts[curport].Invert;
  SetAngleN(curport);  
  changes=true;
//  ShowAngle(curport);
}

//---------------------------------- SetAngle90
void SetAngle90()
{
   AllVar[curport].angle=90;
   Servos[AllPorts[curport].RiseOn].write(AllVar[curport].angle);  //without timer!
//   if (AllPorts[n].frogbud < MAXTERM && AllVar[n].angle==AllVar[n].whoek) SwitchRelay(n);
   dwit();
   Serial.println(F("Angle set to 90 degrees"));
   dgeel();
}
//---------------------------------- CountUp
void CountUp()
{
  if (cmdLH[curport] == 'H') 
  {
    if (AllPorts[curport].Hangle<175) AllPorts[curport].Hangle++;
    AllVar[curport].angle=AllPorts[curport].Hangle;
  } 
  else if (cmdLH[curport] == 'L' && AllPorts[curport].Langle < AllPorts[curport].Hangle-5) 
  {
    AllPorts[curport].Langle++;
    AllVar[curport].angle=AllPorts[curport].Langle;
  }
  else 
  { 
    dblauw();
    Serial.println(F("LOW angle too HIGH"));
    dgeel();
  }
  ShowHL();
}
//---------------------------------- CountDown
void CountDown()
{
  if (cmdLH[curport] == 'L') 
  {
    if (AllPorts[curport].Langle>5) AllPorts[curport].Langle--;
    AllVar[curport].angle=AllPorts[curport].Langle;
  }
  else if (cmdLH[curport] == 'H' && AllPorts[curport].Hangle > AllPorts[curport].Langle+5) 
  {
    AllPorts[curport].Hangle--;
    AllVar[curport].angle=AllPorts[curport].Hangle;
  } 
  else 
  { 
    dblauw();
    Serial.println(F("HIGH angle too LOW"));
    dwit();
  }
  ShowHL();
}
void ShowHL()
{
//Serial.println(Servos[AllPorts[curport].RiseOn].attached());
  AllVar[curport].whoek=(AllPorts[curport].Hangle+AllPorts[curport].Langle)/2;
  Servos[AllPorts[curport].RiseOn].write(AllVar[curport].angle);  //without timer!
  EEPROM.update(A_OUTSET+curport,AllVar[curport].angle);
  changes=true;
  
  AllVar[curport].setpoint=AllVar[curport].angle;
  ShowHoek(curport);
/*
  dwit();
  if (cmdLH[curport] == 'H') Serial.print(F("HIGH angle: "));
  if (cmdLH[curport] == 'L') Serial.print(F("LOW angle: "));
  Serial.print(AllVar[curport].angle);
  Serial.println(F(" degrees"));
  dgeel();
*/
}
//---------------------------------- GetAdmNote
void GetAdmNote()
{
byte inChar=0;
byte nr=0;
char adm[5];

  adm[0]='\0';  //leeg maken
  Serial.print(F("\nEnter administrative note"));
  if (strlen(AllPorts[curport].admin)==0)
	   Serial.print(F(": "));
  else
  {
    Serial.print(F(" ("));
    Serial.print((char*)AllPorts[curport].admin);
    Serial.print(F("): "));
  }
  while (Serial.available()) Serial.read();  // clear inputbuffer
  do
  {
    if (Serial.available()) 
    {
      inChar = Serial.read();
      if (inChar!=13)
      {
        if (inChar==127 && nr>0)  //Backspace
        {
		  nr--;
          adm[nr]='\0';
        }
        else
        {
          if (nr<4) 
          {
			adm[nr]=inChar;
            nr++;
			adm[nr]='\0';  
          }
          else Serial.print(F("\033[1D \033[1D")); //remove invalid character
        }
      }
    }
  } 
  while (inChar != 13);

  dwit();
  Serial.print(F("\n\rAdministrative note "));
  if (nr==0) Serial.println(F("not set/changed"));
  else 
  {
    for (byte i=0;i<5;i++) 
	{
		AllPorts[curport].admin[i]=adm[i];  
		if (AllPorts[curport].frogbud<20) AllPorts[AllPorts[curport].frogbud].admin[i]=adm[i];
	}
    Serial.print(F("set to "));
    Serial.println((char*)AllPorts[curport].admin);
  }
  dgeel();
  changes=true;
}
//---------------------------------- AskInvert servo
void AskInvert()
{
  AllPorts[curport].Invert = 1-AllPorts[curport].Invert;
  dwit(); 
  Serial.print(F("pin on"));
  PortAdres(curport,false);
  if (AllPorts[curport].Invert >0) Serial.println(F(" set to 'Inverted'"));
                            else Serial.println(F(" set to 'not Inverted'"));
   dgeel();
   changes=true;  
}
//---------------------------------- AskFrog
// 1 -> klassiek frog of statusled  
// 2 -> 2 relais optie
void AskFrog()
{
byte n1=0;
byte n2=0;
byte ar=0;
byte curfp;

  curfp=AllPorts[curport].frogbud;
  if (curfp==Val255)  //nieuw
  {
    Serial.print(F("How many relays? ")); 
    ar=AskInteger(0,0,2);
	if (ar>0)
	{
		Serial.print(F("\nSet Frog pin. "));
		n1=AskFrogBud(AllPorts[curport].frogbud);
		if (n1>0)
		{
		  AllPorts[curport].frogbud=n1;
		  AllPorts[n1].frogbud=curport;
		  AllPorts[n1].type=T_Frog1;
		  AllPorts[n1].Invert=false;
		  dwit();	
		  Serial.print(F("\npin "));
		  Serial.print(p[n1]);
		  Serial.println(F(" set as first frog pin")); //eerste pin voor ompoolrelais
		  dgeel();  
		  if (ar==2) 
		  {
			Serial.print(F("Set second Frog pin. "));
			n2=AskFrogBud(AllPorts[curport].frog2);
			if (n2>0)     //tweede pin voor ontkoppel relais beschikbaar
			{
			  AllPorts[curport].frog2=n2;
			  AllPorts[n2].frogbud=curport;
			  AllPorts[n2].type=T_Frog2;
			  AllPorts[n2].Invert=false; 
			  dwit();          
			  Serial.print(F("\npin "));
			  Serial.print(p[n2]);
			  Serial.println(F(" set as second frog pin"));
			  dgeel();
			} 
			else AllPorts[curport].frog2=Val255;
			dgeel();          
		  }  
          changes=true;
		}
	}
// vraag relay inversie indien bestaat
  }
  else
  {
    Serial.print(F("\nRemove frogpoint or Invert relay polarity? R/1/2: "));
    gencmd=GetCommand();
    if (gencmd!='X') changes = true;
	dwit();	
    if (gencmd=='R') 
    {
      ClearFrogBud(curfp);
      if (AllPorts[curport].frog2<MAXTERM) ClearFrogBud(AllPorts[curport].frog2);
      Serial.println(F("Frogpoint removed"));
    }
    else if (gencmd=='1') 
    {
      AllPorts[AllPorts[curport].frogbud].Invert=1-AllPorts[AllPorts[curport].frogbud].Invert;
	  if (AllPorts[AllPorts[curport].frogbud].Invert) Serial.println(F("First Relay inverted"));
	  else Serial.println(F("First Relay NOT inverted"));
    }
    else if (gencmd=='2') if (AllPorts[curport].frog2<Val255) 
    {
	  AllPorts[AllPorts[curport].frog2].Invert = !AllPorts[AllPorts[curport].frog2].Invert;
	  if (AllPorts[AllPorts[curport].frog2].Invert) Serial.println(F("Second Relay inverted"));
	  else Serial.println(F("Second Relay NOT inverted"));
    } else Serial.println(F("No Second Relay"));
	else if (gencmd!='X') ShowInvalid();
	dgeel();
  }
  gencmd='F';
}
//--------------------------- AskFrogBud
byte AskFrogBud(byte fb)   // 1=buddy, 2=frog 1, 3=frog2
{
byte portfb=0;
byte nf=0;
	for (byte n=1;n<MAXTERM;n++) if (AllPorts[n].type==Val255) nf++;
	if (nf==0) 
	{
		dblauw();
		Serial.println(F("\n\rNo more free pins!"));
		dgeel();
		return 0;
	}
//	if (fb>MAXTERM || fb<1)  fb=-1; else {fb=fb+2; if (fb>14) fb++;}
	do
	{
		if (fb>MAXTERM || fb<1) portfb=AskInteger(-1,3,19)-2; else portfb=AskInteger(fb,3,19)-2; 
		if (portfb>12) portfb--;
		
		if (AllPorts[portfb].type<Val255 && portfb !=fb) 
		{
			dblauw();
			Serial.println(F("\npin already in use"));
			dgeel();
			portfb=0;			
		}
		if (portfb==11) {portfb=0; Serial.println("");}
	} while (portfb==0);
	return portfb;
}
//---------------------------- GetBuddy
void GetBuddy(byte curfb)
{
byte s=0;
  Serial.print(F("\nSet Buddy pin. "));
  s=AskFrogBud(curfb);
  if (s>0)
  { 
      AllPorts[curport].frogbud=s;
      AllPorts[s].frogbud=curport;
      AllPorts[s].type=T_Buddy;
	  AllPorts[s].admin[0]='\0';
      changes=true;
      dwit();
      Serial.print(F("\npin "));
      Serial.print(p[AllPorts[curport].frogbud]);
//      Serial.print(F(" set as buddy port to accessory"));
	  Serial.println(F(" set as buddy pin"));
//	  PortAdres(curport,true);
      dgeel();
  } else AllPorts[curport].frogbud=0;

}
//---------------------------------- AskDCC
int AskDCC(byte laag)
{
int adres;
byte sadres;
boolean valid; 
    do
    {
      valid=true;
      Serial.print(F("\nSet DCC address for pin "));
      Serial.println(p[curport]);
//      if (def==0) 
//             adres=AskInteger(-1,1,MAXDCC);       // must enter valid value
      adres=AskInteger(AllPorts[curport].address,laag,MAXDCC);    // may also reset
      if (adres>0)
	  {
		  sadres = chkadres(adres);
//		  Serial.println(sadres);
		  if (sadres>0 && sadres<MAXDCC) 
		  {       
			Serial.print(F("\nAddress in use for pin"));
			PortAdres(sadres,true);
			Serial.print(F("Is that OK ? (Y/N) [N]: "));
			if (AskYN() !=  'Y') valid=false;
		  }		  
	  } else Serial.println("");
    }
    while (!valid);

    if (AllPorts[curport].address!=adres) changes=true;
	AllPorts[curport].address = adres;
	if (adres>0)
	{
		dwit();
		Serial.print(F("\nDCC Address set to "));
		Serial.println(adres); 
		dgeel();
	}
}
//---------------------------------- AskAccMode
void AskAccMode()
{
byte md; 
byte curmd;
boolean enkel=true;
byte curfb;
  curmd=AllPorts[curport].SpeedMode;
  curfb=AllPorts[curport].frogbud;
  do
  {
    valid=true;
    Serial.println(F("\nSelect mode for this accessory"));
    Serial.print(F("0=Help. "));
    do
    {
      if (curmd>9 || curmd<1)
           md=AskInteger(0,0,9);  //nieuwe mode
      else md=AskInteger(curmd,0,9); //bestaande mode
      if (md==0) Serial.println(F(
				 "\n1 Single steady,   2 Double steady     \n\r"
                   "3 Single flashing, 4 Double flashing\n\r"
                   "5 Single One shot, 6 Double One shot\n\r"
                   "7 Analog PWM,      8 Flickering\n\r"
                   "9 Random On/Off\n\r"));
    }
    while (md==0);  
    if ((md==M_AnalPWM || md==M_FirePWM) && pwmports[curport]==0) 
    {
      dblauw();
      Serial.println(F("\nThis pin is no PWM pin"
      "\n\rOnly use pins 3,5,6,or 11 as a PWM pin"));
      dgeel();
      valid=false;
    }
	else
	{
		if (curfb>0 && curfb<MAXTERM) ClearFrogBud(AllPorts[curport].frogbud); //oude buddy eerst opruimen

		if (md==M_DuSteady || md==M_DuFlash  || md==M_DuOneShot)
		{
			GetBuddy(curfb);
			if (AllPorts[curport].frogbud==0) md--; else enkel=false;
		}  
		if (md!=curmd && curmd<10 && curmd>0 && enkel) ResetPort(false);
	}
  } while (!valid);
  
  dwit();
  Serial.print(F("\n\rMode set to "));
  switch (md)
  {
	  case M_SiSteady : Serial.println(F("Single Steady")); break;
	  case M_DuSteady : Serial.println(F("Double Steady")); break;
	  case M_SiFlash  : Serial.println(F("Single Flashing")); break;
	  case M_DuFlash  : Serial.println(F("Double Flashing")); break; 
	  case M_SiOneShot: Serial.println(F("Single One shot")); break;
	  case M_DuOneShot: Serial.println(F("Double One shot")); break;
	  case M_AnalPWM  : Serial.println(F("Analog PWM value")); break;
	  case M_FirePWM  : Serial.println(F("Flickering(PWM)")); break;
	  case M_Random   : Serial.println(F("Random"));
  }
  dgeel();

  {
//	ResetPort(false);
    AllPorts[curport].SpeedMode=md;
    changes=true;
    if (md>=M_SiFlash  && md<=M_DuOneShot) AllPorts[curport].RiseOn=AskOnOffTime((char *)"On",AllPorts[curport].RiseOn);
	if (md==M_SiOneShot) AllPorts[curport].frog2=GetUDB();
    if (md==M_SiFlash  || md==M_DuFlash) AllPorts[curport].FallOffBH=AskOnOffTime((char *)"Off",AllPorts[curport].FallOffBH);
    if (md==M_AnalPWM  || md==M_FirePWM) 
    {
      AllPorts[curport].Langle=AskPWM((char *)"Minimum",AllPorts[curport].Langle);
      AllPorts[curport].Hangle=AskPWM((char *)"Maximum",AllPorts[curport].Hangle);
      if (md==M_AnalPWM)
      {
        AllPorts[curport].RiseOn=AskRiseFall((char *)"Rise",AllPorts[curport].RiseOn);
        AllPorts[curport].FallOffBH=AskRiseFall((char *)"Fall",AllPorts[curport].FallOffBH);
      }
    }
    if (md==M_Random)   //#############
    {
		if (AllPorts[curport].frog2>3) AllPorts[curport].frog2=0;
		AskFix();
		AllPorts[curport].RiseOn=AskRandom((char *)"Minimum",AllPorts[curport].RiseOn);
		AllPorts[curport].FallOffBH=AskRandom((char *)"Maximum",AllPorts[curport].FallOffBH);
    }
  }
}
//---------------------------------- GetUDB
byte GetUDB()
{
char tiepe=' ';
bool valid=false;
	do
	{
		Serial.print(F("\nSelect trigger type Up(U), Down(D) or Both(B): "));
		tiepe=GetCommand();
		if (tiepe=='U'|| tiepe=='D' || tiepe=='B')
		{
			dwit();
			valid=true;
			Serial.print(F("Trigger set to type "));
			if (tiepe=='B') Serial.println(F("Both"));
			else if (tiepe=='U') Serial.println(F("Up-Pulse")); 
			else if (tiepe=='D')Serial.println(F("Down-Pulse")); 
		}
		else ShowInvalid();
	}
	while (!valid);
	dgeel();
	changes=true;
	return tiepe;
}
//---------------------------------- AskPWM
byte AskPWM(char *tekst,byte curval)
{
byte newval;
    Serial.print(F("\nSet "));
    Serial.print(tekst);
    Serial.print(F(" PWM value for pin "));
    PortAdres(curport,true);
    if (tekst=="Maximum") newval=AskInteger(curval,AllPorts[curport].Langle,Val255);  
    if (tekst=="Minimum") newval=AskInteger(curval,0,AllPorts[curport].Hangle);
    dwit();
    Serial.print(F("\n"));
    Serial.print(tekst);
    Serial.print(F(" PWM value set to "));
    Serial.println(newval); 
    dgeel();   
    if (newval != curval) changes=true;
    return newval;    
}
//---------------------------------- AskRiseFall
int AskRiseFall(char *tekst, int curval)
{
int newval;
    Serial.print(F("\nSet "));
    Serial.print(tekst);
    Serial.println(F(" time (sec) for PWM pin "));
    if (curval>1000) 
      newval = AskInteger(-1,0,1000);
    else
      newval = AskInteger(curval,0,1000);
    dwit();
    Serial.print(F("\n"));
    Serial.print(tekst); 
    Serial.print(F(" time set to "));
    Serial.print(newval); 
    Serial.println(F(" sec."));
    dgeel();  
    if (newval != curval) changes=true;   
    return newval; 
}
//---------------------------------- AskRandom
int AskRandom(char *tekst, int curval)
{
 int newval;
 int minval;
 int maxval;

	Serial.print(F("\nSet "));
	Serial.print(tekst);
	Serial.println(F(" random time (x0.02 sec)"));

	if (tekst[1]=='i')  // minimum?
	{
		minval=MINRAND;
		if (AllPorts[curport].FallOffBH==0) maxval=MAXRAND; else maxval=AllPorts[curport].FallOffBH;
		if (curval==0) newval=AskInteger(-1,minval,maxval);
		else newval=AskInteger(curval,minval,maxval);
	}
	else
	{  // maximum
		maxval=MAXRAND;
		if (AllPorts[curport].RiseOn==0) minval=MINRAND; else minval=AllPorts[curport].RiseOn;
		if (curval==0) newval=AskInteger(-1,minval,maxval);
		else newval=AskInteger(curval,minval,maxval);
	}
    dwit();
    Serial.print(F("\n"));
    Serial.print(tekst);
	Serial.print(F(" random time set to "));
    Serial.print(newval*0.02);
    Serial.println(F(" sec."));
    dgeel();   
    if (newval != curval) changes=true;
	return newval; 
}

//---------------------------------- AskFix
void AskFix()
{
//fixed option saved in frog2, the fixed on/off time is stored in BounceLOW

  Serial.println(F("Fixed on/off time? 1=No, 2=ON, 3=OFF"));
  if (AllPorts[curport].frog2>0) 
		 AllPorts[curport].frog2=AskInteger(AllPorts[curport].frog2,1,3);
	else AllPorts[curport].frog2=AskInteger(-1,1,3);
  if (AllPorts[curport].frog2==1) {AllPorts[curport].BounceLOW=0; return;}
  else if (AllPorts[curport].frog2==2) Serial.println (F("\nFixed ON time(x0,02 sec) ?")); 
  else Serial.println (F("\nFixed OFF time(x0,02 sec) ?"));
  
  if (AllPorts[curport].BounceLOW==0) AllPorts[curport].BounceLOW=AskInteger(-1,MINRAND,MAXRAND);
  else AllPorts[curport].BounceLOW=AskInteger(AllPorts[curport].BounceLOW,MINRAND,MAXRAND);
  
  dwit();
  if (AllPorts[curport].frog2==2) Serial.print(F("\nFixed ON time set to ")); 
  else if (AllPorts[curport].frog2==3) Serial.print(F("\nFixed OFF time set to "));
  Serial.print(AllPorts[curport].BounceLOW*0.02);
  Serial.println(F(" sec."));
  dgeel();
}
//---------------------------------- ClearFrogBud
void ClearFrogBud(int fp)
{
//Serial.println("CFB");
  AllPorts[fp].type=Val255;
  AllPorts[fp].frogbud=Val255;
  EEPROM.put(A_PORT+fp*A_GETPUT,AllPorts[fp]);
  if (fp==AllPorts[curport].frogbud) AllPorts[curport].frogbud=Val255;    //remove eigen fp
  if (fp==AllPorts[curport].frog2) AllPorts[curport].frog2=Val255;    //remove eigen fp
}
//---------------------------------- AskOnOffTime
int AskOnOffTime(char *tekst, int curval)
{
int newval;
    Serial.print(F("Enter '"));
    Serial.print(tekst);
    Serial.println(F("' time in millisec."));
    if (curval==0) //no current value
         newval=AskInteger(-1,5,30000);
    else newval=AskInteger(curval,5,30000);
    dwit();
    Serial.print(F("\n'"));
    Serial.print(tekst);
    Serial.print(F("' time set to "));
    Serial.print(newval);
    Serial.println(F(" msec."));
    dgeel();
    if (newval!=curval) changes=true;
    return newval;
}
//------------------------- SetAngleN
void SetAngleN(byte i)
{
  AllVar[i].free=true;
  AllVar[i].savout=1-AllVar[i].output;
  do
  { 
    ProcessServo(i);
  }
  while (!AllVar[i].free);
}
//------------------------- SwitchRelay
void SwitchRelay(byte fp)
{
 if (AllVar[fp].setpoint==AllPorts[fp].Hangle) 
      digitalWrite(p[AllPorts[fp].frogbud],AllPorts[AllPorts[fp].frogbud].Invert);
    else
      digitalWrite(p[AllPorts[fp].frogbud],1-AllPorts[AllPorts[fp].frogbud].Invert);
}
//------------------------- FlushWait 
void FlushWait()
{
  while(Serial.available());  Serial.read();
  while (!Serial.available());   //Wait for new input
}
//-------------------------------- chkadres 
byte chkadres (int adres)
{
  for (byte n=1; n<MAXTERM; n++) 
  {
    if (AllPorts[n].address==adres && n!=curport) return n;
//    if (AllPorts[n].address==adres && AllPorts[n].type==6) return 100;
  }
  return 0;
}
//---------------------------------- AskNumber 
unsigned int AskNumber() 
{
byte ndx = 0;
byte ic = 0;
char receivedChars[10];

    while (Serial.available()) Serial.read();
	while (ic != 13)
	{
		if (Serial.available())
		{
			ic = Serial.read();
			if (ic==127 && ndx>0) 
			{
				ndx--;
				receivedChars[ndx] = '\0';
			}
			else if (isdigit(ic) && ndx<10) 
			{
				receivedChars[ndx] = ic;
				ndx++;
				receivedChars[ndx] = '\0';
			}
		}
	}
    if (ndx==0) return -1; else return atoi(receivedChars);
}
//----------------------------------
unsigned int AskInteger(int curval, int minval, int maxval)
{
unsigned int nr;
bool valid=false;

  while (!valid)
  {
    Serial.print(F("Enter value from "));
    Serial.print(minval);
    Serial.print(F(" to "));
    Serial.print(maxval);
    if (curval>0) 
    {
      Serial.print(F(" ("));
      Serial.print(curval);
      Serial.print(F("): "));
    } else Serial.print(F(": "));

    nr=AskNumber();
    if (nr==-1 && curval>-1) nr=curval; //accept default
    if ((nr>=minval && nr<=maxval) || (nr==curval && nr!=-1)) valid=true;
    else 
    {
      dblauw();
      Serial.println(F("\nNumber not valid!"));
      dgeel();
    }
  }
  return nr;
}
//---------------------------------- AskYN
char AskYN()
{
    FlushWait();
    byte Command = Serial.read();
    if (Command=='y') Command = 'Y'; 
	if (Command=='n') Command = 'N'; 
    if (Command!='Y' && Command!='N') Command =' ';
    Serial.println ("");
    return Command;
}
//------------------------------------
char TestWait(byte n)
{
byte Command=' ';
//	delay (50);
	if (Serial.available()) 
	{ 
		Command = Serial.read();
		if (Command==13) Command='S';
		if (Command>95) Command=Command-32;
		Serial.print ("\r    \r");
		if (Command=='T')
		{
			AllVar[n].output=1-AllVar[n].output; 
			AllVar[n].swrelay=1-AllVar[n].output; 
			DCCval(AllVar[n].output,false);
		}
	}
	return Command;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//=======================================================================
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void NormalMode()
{
byte nc;
  if (Serial.available())
  {
    nc=Serial.read();
    if (nc>95) nc=nc-32;
    if (nc == 'C' || nc == 'P')
    {
      EEPROM.update(MAXMEM,nc); // configuremode
      EEPROM.update(A_RMODE,false); // configuremode
	  EEPROM.update(A_KEEPMODE,true);
      asm volatile ("  jmp 0"); //herstart
    }
  }
//  while(Serial.available()) Serial.read();   //Empty inputbuffer1
  if (LED13state && (millis()-l13mil)>100)
  {
    LED13state=false;
    digitalWrite(LED_BUILTIN,LED13state);
  }
  
  for (byte n=1; n<MAXTERM; n++) 
  {
    if (nextloop) 
    {
      DCC.loop();
      if (AllVar[n].PortWait && (millis()>waitmil[n] + AllVar[n].DelWait)) TogglePort(0,n);
    }
    
    if (AllPorts[n].type==T_Access)     //Accessory
    {
      if (AllPorts[n].SpeedMode==M_SiSteady  || AllPorts[n].SpeedMode==M_DuSteady)  ProcesSteady(n);
      else if (AllPorts[n].SpeedMode==M_SiFlash   || AllPorts[n].SpeedMode==M_DuFlash)   ProcesFlash(n);
      else if (AllPorts[n].SpeedMode==M_SiOneShot || AllPorts[n].SpeedMode==M_DuOneShot) ProcesOneShot(n);
      else if (AllPorts[n].SpeedMode==M_AnalPWM) ProcessPWM(n);
      else if (AllPorts[n].SpeedMode==M_FirePWM) ProcessFire(n);  
      else if (AllPorts[n].SpeedMode==M_Random) ProcesRandom(n);  
    }
    if (AllPorts[n].type==T_Servo && nextloop) ProcessServo(n);
    else if (AllPorts[n].type==T_Input && (millis()-AllVar[n].InpWait>C_INPWAIT) && nextloop) CheckInput(n);
  }
}
//--------------------------------------- CheckInput
void CheckInput(byte i)
{
	byte inp=digitalRead(p[i]);
	if (inp==LastInput[i]) return;		//No change
    if (inp>0) inp=1;
	LastInput[i]=inp;
	
	for (byte x=0;x<MAXTERM;x++) 
	{
		if (AllPorts[x].type==T_Access || AllPorts[x].type==T_Servo)
		{
			if (AllPorts[x].address==AllPorts[i].address)   //Acc of servo op eigen adres
			{  //found acc or servo on other port
				if (AllPorts[i].frogbud=='B') //SetPort (i,x); //follow input
				{
					Serial.print(F("Level on pin "));
					Serial.print(p[i]);
          // Added pin number
          //Serial.print(" (pin "); Serial.print(p[i]); Serial.print(")");
					Serial.print(F(" set to "));
					if (inp==LOW) Serial.println(F("Low")); else Serial.println(F("High"));
					AllVar[x].output=AllPorts[i].Invert^inp; //@@@@@@@@@@@@@@@@@@@@@@@
					AllVar[x].swrelay=1-AllVar[x].output;
					EEPROM.update(A_OUTSET+x,AllVar[x].output);
				}
				if (AllPorts[i].frogbud=='D' && inp==LOW) 
				{
					TogglePort(i,x);
					if (AllPorts[x].type==T_Access && AllPorts[x].SpeedMode==M_SiOneShot)
						InpSingle=true; else InpSingle=false;  //forceer oneshot bij U en D bij input
				}
				if (AllPorts[i].frogbud=='U' && inp==HIGH) TogglePort(i,x);
			}
			if (AllPorts[i].frogbud=='B'|| 
			   (AllPorts[i].frogbud=='D' && inp==LOW) ||
			   (AllPorts[i].frogbud=='U' && inp==HIGH))
			{
				if (AllPorts[x].address==AllPorts[i].RiseOn) SetWait(AllPorts[i].Langle,x); //dcc2
				if (AllPorts[x].address==AllPorts[i].FallOffBH) SetWait(AllPorts[i].Hangle,x); //dcc3
			} 				
		}
	}

    AllVar[i].InpWait=millis(); 		
}
void SetWait (byte wait, byte x)
{
	waitmil[x]=millis();
	AllVar[x].DelWait=wait*100; //busy waiting
	AllVar[x].PortWait=true;
}

//----------------
void TogglePort (byte i, byte j)
{
	if (i>0)
	{
		if (AllPorts[i].frogbud=='D') Serial.print(F("Down"));
		if (AllPorts[i].frogbud=='U') Serial.print(F("Up"));
		Serial.print(F(" pulse detected on pin "));
    // removed line feed
		Serial.print(i);
    // added pin number
    Serial.print(" (pin "); Serial.print(p[i]); Serial.println(")");
	}
	AllVar[j].output=1-AllVar[j].output;
	AllVar[j].swrelay=1-AllVar[j].output;
	EEPROM.update(A_OUTSET+j,AllVar[j].output);	
	if (AllVar[j].DelWait>0)
	{
		Serial.print(F("Waited "));
		Serial.print(AllVar[j].DelWait);
		Serial.print(F(" msec. for pin "));
		Serial.println(j);
    }
	AllVar[j].DelWait=0;
	AllVar[j].PortWait=false;
    l13mil=millis(); 
    LED13state=true;
    digitalWrite(LED_BUILTIN,LED13state);
}
//--------------------------------------- ProcessFire pwm
void ProcessFire(byte n)
{
	if (AllVar[n].output==AllVar[n].swrelay)
	{
		if ((AllVar[n].setpoint==1) && (millis() > AllVar[n].lastmillis+30))
		{
			analogWrite(p[n],random(AllPorts[n].Langle,AllPorts[n].Hangle));
			AllVar[n].lastmillis=millis();
		}
		return;
	}
		
	if (Rmode) 
	{
		ShowAccDCC(n);
		EEPROM.update(A_OUTSET+n,AllVar[n].output);
	}
	
	if (AllVar[n].output^AllPorts[n].Invert==0)   // no flickering --> stoppen
	{
		digitalWrite(p[n],LOW^AllPorts[n].Invert); //led uit
		AllVar[n].setpoint=0;   //no flickering busy
		if (Rmode) Serial.println(F(" stopped flickering"));
	} 
 
	if (AllVar[n].output^AllPorts[n].Invert==1)        // Flashing
	{
		AllVar[n].setpoint=1;	//flashing busy
		if (Rmode) Serial.println(F(" started flickering"));
	}

    AllVar[n].swrelay = AllVar[n].output;
}
//-------------
/*
fire


//byte val;
  if (AllVar[n].output==0^AllPorts[n].Invert)   //Uit
  {
	if (Rmode)
	{
		if (notice[n]) 
		{
			ShowAccDCC(n);
			Serial.println(F(" stops flickering")); 
			notice[n]=false;
			EEPROM.update(A_OUTSET+n,AllVar[n].output); 
		}
    }
	analogWrite(p[n],0);
  }
  
  if (AllVar[n].output==1^AllPorts[n].Invert)   	//Aan
  {
    if (!notice[n] && Rmode) 
    {
      ShowAccDCC(n);
      Serial.println(F(" starts flickering"));
      notice[n]=true; 
      EEPROM.update(A_OUTSET+n,AllVar[n].output);      
    }
    if (millis() > AllVar[n].lastmillis+20)
    {
//	  val=random(AllPorts[n].Langle,AllPorts[n].Hangle);
//	  Serial.println(val);
      analogWrite(p[n],random(AllPorts[n].Langle,AllPorts[n].Hangle));
      AllVar[n].lastmillis=millis();
    }
  }
}
*/
//-------------------------------------------- ProcessPWM
void ProcessPWM (byte n)
{
	if (!nextloop && Rmode) return;
	if (AllVar[n].free)		//Niet bezig
	{
		if (AllVar[n].swrelay != AllVar[n].output)  // output changed
		{   
			ShowAccDCC(n);
			AllVar[n].swrelay = AllVar[n].output;
			
			AllVar[n].lastmillis=millis(); 
			
			if (AllVar[n].output^AllPorts[n].Invert==1)  //stijgen
			{
				if (AllPorts[n].RiseOn==0)   //stijgtijd=0
				{
					analogWrite(p[n],AllPorts[n].Hangle);   // direct maximaal
					Serial.println(F(" set to maximum value"));
					Serial.println("3");
				}			
				else
				{
					steptime[pwm[n]]=AllPorts[n].RiseOn*1000/(AllPorts[n].Hangle-AllPorts[n].Langle);  //stijgtijd/aantal stappen 
					Serial.println(F(" starts rising"));
					AllVar[n].free=false; //niet klaar
					updown[pwm[n]]=1;             // we gaan omhoog				
					curpwm[pwm[n]]=AllPorts[n].Langle;         //beginnen bij laag		
					analogWrite(p[n],curpwm[pwm[n]]); //hoger/lager						
				}
			}
			if (AllVar[n].output^AllPorts[n].Invert==0)  //dalen
			{
				if (AllPorts[n].FallOffBH==0)
				{
					analogWrite(p[n],AllPorts[n].Langle);   // direct stoppen
					Serial.println(F(" set to minimum value"));
				}
				else
				{
					steptime[pwm[n]]=AllPorts[n].FallOffBH*1000/(AllPorts[n].Hangle-AllPorts[n].Langle);  //daaltijd/aantal stappen 
					Serial.println(F(" starts falling"));
					AllVar[n].free=false; 			//niet klaar
					updown[pwm[n]]=-1;             	// we gaan omlaag				
					curpwm[pwm[n]]=AllPorts[n].Hangle;             //beginnen bij hoog
					analogWrite(p[n],curpwm[pwm[n]]); //hoger/lager						
				}
			}
		}
//		if (AllPorts[n].Invert) updown[pwm[n]]=updown[pwm[n]]*-1;
	}
	else		//Wel bezig
	{
		if (!AllVar[n].free && millis()>(AllVar[n].lastmillis + steptime[pwm[n]]))  //next step
		{
		    AllVar[n].lastmillis=millis(); 
			curpwm[pwm[n]]=curpwm[pwm[n]]+updown[pwm[n]];
			analogWrite(p[n],curpwm[pwm[n]]); //hoger/lager
//Serial.println(curpwm[pwm[n]]);
			if ((curpwm[pwm[n]]==AllPorts[n].Langle)||(curpwm[pwm[n]]==AllPorts[n].Hangle)) //Klaar?
			{  
			  AllVar[n].free=true;
			  ShowAccDCC(n);
			  Serial.println(F(" stops rising/falling"));
			}
		}
	}
}

//--------------------------------------- ProcesSteady
void ProcesSteady (byte n)
{
    if (AllVar[n].swrelay != AllVar[n].output)    // output changed
    { 
      digitalWrite(p[n],AllPorts[n].Invert^AllVar[n].output);
      if (AllPorts[n].SpeedMode==M_DuSteady) digitalWrite(p[AllPorts[n].frogbud],AllPorts[n].Invert^(1-AllVar[n].output));
      AllVar[n].swrelay = AllVar[n].output;
	  if (Rmode)
	  {
		ShowAccDCC(n);
		Serial.print(F(" is turned"));
		if (AllVar[n].output==0^AllPorts[n].Invert) Serial.println(F(" off")); else Serial.println(F(" on"));
		EEPROM.update(A_OUTSET+n,AllVar[n].output);  
	  }
    }
}
//--------------------------------------- ProcesFlash
void ProcesFlash (byte n)
{
	if (AllVar[n].output==AllVar[n].swrelay)
	{
		if (AllVar[n].setpoint==1) //BEZIG MET FLASHEN
		{
		    if (millis()>AllVar[n].lastmillis+AllPorts[n].RiseOn && !AllVar[n].free) SwitchFlash(n);    
			if (millis()>AllVar[n].lastmillis+AllPorts[n].FallOffBH && AllVar[n].free) SwitchFlash(n);
		}
		return;
	}
	if (Rmode) 
	{
		ShowAccDCC(n);
		EEPROM.update(A_OUTSET+n,AllVar[n].output);
	}
	if (AllVar[n].output^AllPorts[n].Invert==0)   // no flashing --> stoppen
	{
		digitalWrite(p[n],LOW^AllPorts[n].Invert); //led uit
		if (AllPorts[n].SpeedMode==M_DuFlash) digitalWrite(p[AllPorts[n].frogbud], LOW^AllPorts[n].Invert);
		AllVar[n].setpoint=0;   //no flashing busy
		if (Rmode) Serial.println(F(" stopped flashing"));
	} 

	if (AllVar[n].output^AllPorts[n].Invert==1)        // Flashing
	{
		AllVar[n].setpoint=1;	//flashing busy
		if (Rmode) Serial.println(F(" started flashing"));
	}

    AllVar[n].swrelay = AllVar[n].output;
}
//-------------
void SwitchFlash(byte n)
{
  digitalWrite(p[n],AllVar[n].free^AllPorts[n].Invert);
  if (AllPorts[n].SpeedMode==M_DuFlash) digitalWrite(p[AllPorts[n].frogbud],!AllVar[n].free^AllPorts[n].Invert);
  AllVar[n].free=1-AllVar[n].free;
  AllVar[n].lastmillis=millis();
}
//--------------------------------------- ProcesRandom
void ProcesRandom (byte port)
{
static unsigned long randtime[MAXTERM];
int mf=20;
  if (AllVar[port].output==0 && AllVar[port].swrelay != AllVar[port].output)   // no on/off 
  {
      digitalWrite(p[port],AllPorts[port].Invert^LOW); 
	  if (Rmode)
	  {
		ShowAccDCC(port);
		Serial.println(F(" random off"));
        EEPROM.update(A_OUTSET+port,AllVar[port].output);
	  }
  }
  
  if (AllVar[port].output==1)        // random on/off
  {
    if (AllVar[port].swrelay==0) // net gewisseld
    {
	  if (Rmode)
	  {
		ShowAccDCC(port);
		Serial.println(F(" random on"));
		EEPROM.update(A_OUTSET+port,AllVar[port].output);
	  }
      randtime[port]=0;
      AllVar[port].free=true;
      AllVar[port].lastmillis=millis();
    }   
    if (millis() > (AllVar[port].lastmillis + randtime[port])) 
    {
      AllVar[port].lastmillis=millis();
      digitalWrite(p[port],AllPorts[port].Invert^!AllVar[port].free);
      AllVar[port].free=!AllVar[port].free; //free true -> on, free false -> off
	  randtime[port]=mf*random(AllPorts[port].RiseOn,AllPorts[port].FallOffBH);	  
	  if (AllPorts[port].frog2==2 && AllVar[port].free) randtime[port]=mf*AllPorts[port].BounceLOW;
	  if (AllPorts[port].frog2==3 &&!AllVar[port].free) randtime[port]=mf*AllPorts[port].BounceLOW;
//	  Serial.println(randtime[port]);
    }
  }
  AllVar[port].swrelay = AllVar[port].output;
}
//-------------------------------------------- ProcesOneShot
void ProcesOneShot (byte n) 
{
  if (AllVar[n].swrelay != AllVar[n].output)    // output changed
  {
    if (AllVar[n].free) // No shot busy, Start one shot
    {
		if (AllPorts[n].SpeedMode==M_SiOneShot)
		{
			   if (AllVar[n].output==1 && (AllPorts[n].frog2=='U' || AllPorts[n].frog2=='B')) Shoot(n,0);
		  else if (AllVar[n].output==0 && (AllPorts[n].frog2=='D' || AllPorts[n].frog2=='B')) Shoot(n,0);
		  else if (AllVar[n].output==0 && InpSingle==true) Shoot(n,0);
	    }
		if (AllPorts[n].SpeedMode==M_DuOneShot) 
		{
			if (AllVar[n].output==1) Shoot(n,0); else Shoot(n,AllPorts[n].frogbud);
		}
        AllVar[n].swrelay=AllVar[n].output;
		AllVar[n].lastmillis=millis();
    }
  }

  if (!AllVar[n].free && millis() > AllVar[n].lastmillis+AllPorts[n].RiseOn)  //OverShoot ?
  {
    digitalWrite(p[n],AllPorts[n].Invert^LOW);
    if (AllPorts[n].frogbud<MAXTERM) digitalWrite(p[AllPorts[n].frogbud],AllPorts[n].Invert^LOW);
    AllVar[n].free=true;

// TogglePort accessory with same address
    if (AllPorts[n].SpeedMode==M_SiOneShot && Rmode)
    {
      for (byte i=0;i<MAXTERM;i++) 
      {
        if (AllPorts[i].address==AllPorts[n].address && n != i && AllPorts[i].type==T_Access) 
        {
          AllVar[i].output=1-AllVar[i].output;
          Serial.print(F("Changed pin "));
          Serial.println(i);
        }
      }
    }
  }
}
//-------------------------------------------
void Shoot(byte n,byte buddy)
{
  if (buddy==0) 
  { 
    digitalWrite(p[n] ,AllPorts[n].Invert^HIGH); 
    if (Rmode) ShowAccDCC(n);
  }
  else 
  {
    digitalWrite(p[buddy],AllPorts[n].Invert^HIGH);
    if (Rmode) ShowAccDCC(buddy);
  }
  if (Rmode) Serial.println(F(" fired one shot"));
  AllVar[n].free=false;
  AllVar[n].lastmillis=millis();
  AllVar[n].swrelay = AllVar[n].output;  //save position
}
//-------------------------------------------- ShowAccDCC
void ShowAccDCC(byte n)
{
  Serial.print(F("Accessory "));
  Serial.print(AllPorts[n].admin);
  PortAdres(n,false);
}
//-------------------------------------------- ProcessServo
void ProcessServo (byte n)
{
static byte bouncecnt[MAXTERM];
static byte HL;
  if (AllVar[n].savout != AllVar[n].output && AllVar[n].free)    // output changed --> prepare move
  {
    if (AllVar[n].output)      // set new setpoint
    {
      if (AllPorts[n].Invert) 
           AllVar[n].setpoint=AllPorts[n].Hangle;
      else AllVar[n].setpoint=AllPorts[n].Langle;
    }
    else
    {
      if (AllPorts[n].Invert) 
           AllVar[n].setpoint=AllPorts[n].Langle;
      else AllVar[n].setpoint=AllPorts[n].Hangle;
    }
//	Serial.println(AllVar[n].setpoint);
	bouncecnt[n]=0;
	AllVar[n].doel = AllVar[n].setpoint;
	if (AllVar[n].doel==AllPorts[n].Langle) HL=0;
	else if (AllVar[n].doel==AllPorts[n].Hangle) HL=1;

    if (AllPorts[n].frog2<MAXTERM) digitalWrite(p[AllPorts[n].frog2],AllPorts[AllPorts[n].frog2].Invert); // ontkoppel puntstuk
    if (Rmode) Servos[AllPorts[n].RiseOn].attach(p[n]);       // Attach servo
    AllVar[n].free = false;             // indicate the junction is moving    
    AllVar[n].savout = AllVar[n].output; //save state
  }

  if (AllVar[n].angle!=AllVar[n].setpoint && millis() > AllVar[n].lastmillis+AllPorts[n].SpeedMode)  //not ready yet
  {
    if (AllVar[n].angle < AllVar[n].setpoint) AllVar[n].angle++;  // Raise angle if required
    if (AllVar[n].angle > AllVar[n].setpoint) AllVar[n].angle--;  // Decrease angle if required 

    Servos[AllPorts[n].RiseOn].write(AllVar[n].angle);    // Move servo 1 degree towards required setpoint
//Serial.println(AllVar[n].angle);
    if (AllPorts[n].frogbud < MAXTERM && AllVar[n].angle==AllVar[n].whoek) SwitchRelay(n);
    if (AllVar[n].angle==AllVar[n].setpoint)    //klaar!
    {
      if ((AllPorts[n].BounceLOW==0 && HL==0) || (AllPorts[n].FallOffBH==0 && HL==1)) AllVar[n].free=true;
      else 
      {
		if (AllVar[n].doel == AllVar[n].setpoint)  //op eindpunt, naar volgende bounce?
		{
			while (bouncecnt[n]<4 && BounceHL[AllPorts[n].RiseOn][bouncecnt[n]][HL]==0) bouncecnt[n]++; 
			if (bouncecnt[n]==4) AllVar[n].free=true; //dan klaar
			else 
			{
				if (BounceHL[AllPorts[n].RiseOn][bouncecnt[n]][HL]>0)
				{
					if (AllVar[n].angle==AllPorts[n].Langle) 
					  AllVar[n].setpoint=AllVar[n].setpoint + BounceHL[AllPorts[n].RiseOn][bouncecnt[n]][HL];   // naar volgende bouncehoek
					else 
					  AllVar[n].setpoint=AllVar[n].setpoint - BounceHL[AllPorts[n].RiseOn][bouncecnt[n]][HL];   // naar volgende bouncehoek	
				}				
			    bouncecnt[n]++;  // volgende klaar zetten.   
			}
		}
		else AllVar[n].setpoint=AllVar[n].doel;   //op bouncehoek --> terug naar doel
		
//Serial.println(AllVar[n].setpoint);		
     }	  
	}
	if (AllVar[n].free)
	{		
      if (AllVar[n].setpoint==AllPorts[n].Langle) cmdLH[n]='L'; else cmdLH[n]='H';
//      AllVar[n].free = true;             // Indicate the junction is in position.
      if (Rmode && deta) Servos[AllPorts[n].RiseOn].detach();       // Detach to prevent jittering    
      if (AllPorts[n].frog2<MAXTERM) digitalWrite(p[AllPorts[n].frog2],!AllPorts[AllPorts[n].frog2].Invert); // koppel puntstuk
      EEPROM.update(A_OUTSET+n,AllVar[n].setpoint);      
	  ShowHoek(n);

    } 
    else AllVar[n].lastmillis=millis();     // Save last changetime  
  }    
}
//----------------------
void ShowHoek(byte n)
{
//	curport=n;
//	ShowHL();
	dwit();
	  Serial.print(F("Servo "));
///    Serial.print(F("Servo set to "));
      PortAdres(n,false);
//	  Serial.print(AllPorts[n].admin);
//	  Serial.print(F(" on A"));
//    Serial.print(AllPorts[n].address);
//	  Serial.print(F("/P"));
//	  Serial.print(n);
      Serial.print(F(" set to "));
      Serial.print(AllVar[n].setpoint);
      Serial.println(F(" degrees"));
	dgeel();
}
//---------------- End Of Program ----------------------
